/**
 *
 * This file is part of Tulip (www.tulip-software.org)
 *
 * Authors: David Auber, Patrick Mary, Morgan Mathiaut
 * from the LaBRI Visualization Team
 * Last modification : 15/01/2010
 *
 * Tulip is free software; you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * Tulip is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 */

#ifndef _StrongComponent_H
#define _StrongComponent_H
#include <stack>
#include <tulip/tuliphash.h>
#include <tulip/TulipPlugin.h>
struct NodeInfo{
  NodeInfo(int stra=0,int sta=0):prefixOrder(stra),minAttach(sta){}
  int prefixOrder;
  int minAttach;
};

/// StrongComponent.h - An implementation of the Stongly connected component algorithm.
/** This plugin is an implementation of the Stongly connected algorithm.
 *
 *  <b>HISTORY</b>
 *
 *  - 01/12/1999 Verson 0.0.1: Initial release
 *
 *  \note This algorithm assigns to each node a value defined as following : If two nodes are in the same
 *  strongly connected component they have the same value else they have a 
 *  different value.
 *
 *  \author David Auber University Bordeaux I France: Email:auber@tulip-software.com
 *
 *  <b>LICENCE</b>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or     
 *  (at your option) any later version.
 *
 */
class StrongComponent:public tlp::DoubleAlgorithm { 
public:
  StrongComponent(const tlp::PropertyContext &);
  ~StrongComponent();
  bool run();
  bool check(std::string &);
  void reset();
  
private:
  int  attachNumerotation(tlp::node, TLP_HASH_MAP<tlp::node,bool> &,
			  TLP_HASH_MAP<tlp::node,bool> &,
			  TLP_HASH_MAP<tlp::node,int> &, int &,
			  std::stack<tlp::node> &, int &);
};

#endif











